@@ -27,7 +27,7 @@ UFC function from an (optimized) intermediate representation (OIR).
 # Modified by Martin Alnaes, 2013
 #
 # First added:  2009-12-16
-# Last changed: 2013-02-10
+# Last changed: 2014-03-19
 
 # FFC modules
 from ffc.log import info, begin, end, debug_code
@@ -51,7 +51,6 @@ def _not_implemented(function_name, return_null=False):
         body += "\n" + format["return"](0)
     return body
 
-
 def generate_code(ir, prefix, parameters):
     "Generate code from intermediate representation."
 
@@ -214,31 +213,42 @@ def _generate_form_code(ir, prefix, parameters):
     code = {}
     code["classname"] = classname(prefix, ir["id"])
     code["members"] = ""
+
     code["constructor"] = do_nothing
     code["constructor_arguments"] = ""
     code["initializer_list"] = ""
     code["destructor"] = do_nothing
+
     code["signature"] = ret('"%s"' % ir["signature"])
     code["rank"] = ret(ir["rank"])
     code["num_coefficients"] = ret(ir["num_coefficients"])
+
     code["num_cell_domains"] = ret(ir["num_cell_domains"])
     code["num_exterior_facet_domains"] = ret(ir["num_exterior_facet_domains"])
     code["num_interior_facet_domains"] = ret(ir["num_interior_facet_domains"])
     code["num_point_domains"] = ret(ir["num_point_domains"])
+    code["num_custom_domains"] = ret(ir["num_custom_domains"])
+
     code["has_cell_integrals"] = _has_foo_integrals(ir, "cell")
     code["has_exterior_facet_integrals"] = _has_foo_integrals(ir, "exterior_facet")
     code["has_interior_facet_integrals"] = _has_foo_integrals(ir, "interior_facet")
     code["has_point_integrals"] = _has_foo_integrals(ir, "point")
+    code["has_custom_integrals"] = _has_foo_integrals(ir, "custom")
+
     code["create_finite_element"] = _create_foo(prefix, "finite_element", ir["create_finite_element"])
     code["create_dofmap"] = _create_foo(prefix, "dofmap", ir["create_dofmap"])
+
     code["create_cell_integral"] = _create_foo_integral(ir, "cell", prefix)
     code["create_exterior_facet_integral"] = _create_foo_integral(ir, "exterior_facet", prefix)
     code["create_interior_facet_integral"] = _create_foo_integral(ir, "interior_facet", prefix)
     code["create_point_integral"] = _create_foo_integral(ir, "point", prefix)
+    code["create_custom_integral"] = _create_foo_integral(ir, "custom", prefix)
+
     code["create_default_cell_integral"] = _create_default_foo_integral(ir, "cell", prefix)
     code["create_default_exterior_facet_integral"] = _create_default_foo_integral(ir, "exterior_facet", prefix)
     code["create_default_interior_facet_integral"] = _create_default_foo_integral(ir, "interior_facet", prefix)
     code["create_default_point_integral"] = _create_default_foo_integral(ir, "point", prefix)
+    code["create_default_custom_integral"] = _create_default_foo_integral(ir, "custom", prefix)
 
     # Postprocess code
     _postprocess_code(code, parameters)
@@ -465,10 +475,10 @@ def _create_foo_integral(ir, integral_type, prefix):
     postfix = ir["create_" + integral_type + "_integral"]
     return _create_foo(prefix, class_name, postfix, numbers=postfix)
 
-def _has_foo_integrals(ir, domain_type):
+def _has_foo_integrals(ir, integral_type):
     ret = format["return"]
     b = format["bool"]
-    i = ir["has_%s_integrals" % domain_type]
+    i = ir["has_%s_integrals" % integral_type]
     return ret(b(i))
 
 def _create_default_foo_integral(ir, integral_type, prefix):
