@@ -22,7 +22,7 @@
 # Modified by Martin Alnaes 2013
 #
 # First added:  2009-12-16
-# Last changed: 2013-01-25
+# Last changed: 2014-04-15
 
 # Python modules
 import re, numpy, platform
@@ -31,7 +31,8 @@ import re, numpy, platform
 from ffc.log import debug, error
 
 # Mapping of restrictions
-_choose_map = {None: "", "+": "_0", "-": "_1"}
+_fixed_map = {None: "", "+": "_0", "-": "_1"}
+_choose_map = lambda r: _fixed_map[r] if r in _fixed_map else "_%s" % str(r)
 
 # FIXME: MSA: Using a dict to collect functions in a namespace is weird
 #             and makes the code harder to follow, change to a class
@@ -75,9 +76,14 @@ format.update({
     "uint declaration":               "unsigned int",
     "static const uint declaration":  "static const unsigned int",
     "static const float declaration": "static const double",
+    "vector table declaration":       "std::vector< std::vector<double> >",
+    "double array declaration":       "double*",
+    "const double array declaration": "const double*",
     "const float declaration":        lambda v, w: "const double %s = %s;" % (v, w),
     "const uint declaration":         lambda v, w: "const unsigned int %s = %s;" % (v, w),
     "dynamic array":                  lambda t, n, s: "%s *%s = new %s[%s];" % (t, n, t, s),
+    "static array":                   lambda t, n, s: "static %s %s[%d];" % (t, n, s),
+    "fixed array":                    lambda t, n, s: "%s %s[%d];" % (t, n, s),
     "delete dynamic array":           lambda n, s=None: _delete_array(n, s),
     "create foo":                     lambda v: "new %s()" % v
 })
@@ -132,21 +138,21 @@ format.update({
     "cell":               lambda s: "ufc::%s" % s,
     "J":                  lambda i, j, m, n: "J[%d]" % _flatten(i, j, m, n),
     "inv(J)":             lambda i, j, m, n: "K[%d]" % _flatten(i, j, m, n),
-    "det(J)":             lambda r=None: "detJ%s" % _choose_map[r],
-    "cell volume":        lambda r=None: "volume%s" % _choose_map[r],
-    "circumradius":       lambda r=None: "circumradius%s" % _choose_map[r],
+    "det(J)":             lambda r=None: "detJ%s" % _choose_map(r),
+    "cell volume":        lambda r=None: "volume%s" % _choose_map(r),
+    "circumradius":       lambda r=None: "circumradius%s" % _choose_map(r),
     "facet area":         "facet_area",
     "min facet edge length": lambda r: "min_facet_edge_length",
     "max facet edge length": lambda r: "max_facet_edge_length",
     "scale factor":       "det",
     "transform":          lambda t, i, j, m, n, r: _transform(t, i, j, m, n, r),
-    "normal component":   lambda r, j: "n%s%s" % (_choose_map[r], j),
+    "normal component":   lambda r, j: "n%s%s" % (_choose_map(r), j),
     "x coordinate":       "X",
     "y coordinate":       "Y",
     "z coordinate":       "Z",
     "ip coordinates":     lambda i, j: "X%d[%d]" % (i, j),
     "affine map table":   lambda i, j: "FEA%d_f%d" % (i, j),
-    "vertex_coordinates": lambda r=None: "vertex_coordinates%s" % _choose_map[r]
+    "vertex_coordinates": lambda r=None: "vertex_coordinates%s" % _choose_map(r)
 })
 
 # UFC function arguments and class members (names)
@@ -158,7 +164,7 @@ format.update({
     "argument derivative order":  "n",
     "argument values":            "values",
     "argument coordinates":       "dof_coordinates",
-    "facet":                      lambda r: "facet%s" % _choose_map[r],
+    "facet":                      lambda r: "facet%s" % _choose_map(r),
     "vertex":                     "vertex",
     "argument axis":              "i",
     "argument dimension":         "d",
@@ -187,26 +193,27 @@ format.update({
 # code generators.
 format.update({
     # evaluate_basis and evaluate_basis_derivatives
-    "tmp value":                lambda i: "tmp%d" % i,
-    "tmp ref value":            lambda i: "tmp_ref%d" % i,
-    "local dof":                "dof",
-    "basisvalues":              "basisvalues",
-    "coefficients":             lambda i: "coefficients%d" %(i),
-    "num derivatives":          lambda t_or_g :"num_derivatives" + t_or_g,
-    "derivative combinations":  lambda t_or_g :"combinations" + t_or_g,
-    "transform matrix":         "transform",
-    "transform Jinv":           "Jinv",
-    "dmats":                    lambda i: "dmats%s" %(i),
-    "dmats old":                "dmats_old",
-    "reference derivatives":    "derivatives",
-    "dof values":               "dof_values",
-    "dof map if":               lambda i,j: "%d <= %s && %s <= %d"\
-                                % (i, format["argument basis num"], format["argument basis num"], j),
-    "dereference pointer":      lambda n: "*%s" % n,
-    "reference variable":       lambda n: "&%s" % n,
-    "call basis":               lambda i, s: "evaluate_basis(%s, %s, x, vertex_coordinates, cell_orientation);" % (i, s),
-    "call basis all":           "evaluate_basis_all(values, x, vertex_coordinates, cell_orientation);",
-    "call basis_derivatives":   lambda i, s: "evaluate_basis_derivatives(%s, n, %s, x, vertex_coordinates, cell_orientation);" % (i, s),
+    "tmp value":                  lambda i: "tmp%d" % i,
+    "tmp ref value":              lambda i: "tmp_ref%d" % i,
+    "local dof":                  "dof",
+    "basisvalues":                "basisvalues",
+    "coefficients":               lambda i: "coefficients%d" %(i),
+    "num derivatives":            lambda t_or_g :"num_derivatives" + t_or_g,
+    "derivative combinations":    lambda t_or_g :"combinations" + t_or_g,
+    "transform matrix":           "transform",
+    "transform Jinv":             "Jinv",
+    "dmats":                      lambda i: "dmats%s" %(i),
+    "dmats old":                  "dmats_old",
+    "reference derivatives":      "derivatives",
+    "dof values":                 "dof_values",
+    "dof map if":                 lambda i,j: "%d <= %s && %s <= %d"\
+                                  % (i, format["argument basis num"], format["argument basis num"], j),
+    "dereference pointer":        lambda n: "*%s" % n,
+    "reference variable":         lambda n: "&%s" % n,
+    "call basis":                 lambda i, s: "_evaluate_basis(%s, %s, x, vertex_coordinates, cell_orientation);" % (i, s),
+    "call basis_all":             "_evaluate_basis_all(values, x, vertex_coordinates, cell_orientation);",
+    "call basis_derivatives":     lambda i, s: "_evaluate_basis_derivatives(%s, n, %s, x, vertex_coordinates, cell_orientation);" % (i, s),
+    "call basis_derivatives_all": lambda i, s: "_evaluate_basis_derivatives_all(n, %s, x, vertex_coordinates, cell_orientation);" % s,
 
     # quadrature code generators
     "integration points": "ip",
@@ -222,11 +229,12 @@ format.update({
 #    "basis constant":     lambda i: "B%d" % i,
     "function value":     lambda i: "F%d" % i,
     "nonzero columns":    lambda i: "nzc%d" % i,
-    "weight":             lambda i: "W%d" % (i),
+    "weight":             lambda i: "W" if i is None else "W%d" % (i),
     "psi name":           lambda c, et, e, co, d, a: _generate_psi_name(c, et, e, co, d, a),
     # both
     "free indices":       ["r","s","t","u"],
-    "matrix index":       lambda i, j, range_j: _matrix_index(i, str(j), str(range_j))
+    "matrix index":       lambda i, j, range_j: _matrix_index(i, str(j), str(range_j)),
+    "quadrature point":   lambda i, gdim: "quadrature_points + %s*%d" % (i, gdim)
 })
 
 # Misc
@@ -245,31 +253,39 @@ from codesnippets import *
 
 format.update({
     "compute_jacobian":         lambda tdim, gdim, r=None: \
-                                compute_jacobian[tdim][gdim] % {"restriction": _choose_map[r]},
+                                compute_jacobian[tdim][gdim] % {"restriction": _choose_map(r)},
     "compute_jacobian_inverse": lambda tdim, gdim, r=None: \
-                                compute_jacobian_inverse[tdim][gdim] % {"restriction": _choose_map[r]},
-    "orientation":              lambda tdim, gdim, r=None: orientation_snippet % {"restriction": _choose_map[r]} if tdim != gdim else "",
-    "facet determinant":        lambda tdim, gdim, r=None: facet_determinant[tdim][gdim] % {"restriction": _choose_map[r]},
+                                compute_jacobian_inverse[tdim][gdim] % {"restriction": _choose_map(r)},
+    "orientation":              lambda tdim, gdim, r=None: orientation_snippet % {"restriction": _choose_map(r)} if tdim != gdim else "",
+    "facet determinant":        lambda tdim, gdim, r=None: facet_determinant[tdim][gdim] % {"restriction": _choose_map(r)},
     "fiat coordinate map":      lambda cell, gdim: fiat_coordinate_map[cell][gdim],
     "generate normal":          lambda tdim, gdim, i: _generate_normal(tdim, gdim, i),
-    "generate cell volume":     lambda tdim, gdim, i: _generate_cell_volume(tdim, gdim, i),
-    "generate circumradius":    lambda tdim, gdim, i: _generate_circumradius(tdim, gdim, i),
+    "generate cell volume":     lambda tdim, gdim, i, r=None: _generate_cell_volume(tdim, gdim, i, r),
+    "generate circumradius":    lambda tdim, gdim, i, r=None: _generate_circumradius(tdim, gdim, i, r),
     "generate facet area":      lambda tdim, gdim: facet_area[tdim][gdim],
-    "generate min facet edge length": lambda tdim, gdim, r=None: min_facet_edge_length[tdim][gdim] % {"restriction": _choose_map[r]},
-    "generate max facet edge length": lambda tdim, gdim, r=None: max_facet_edge_length[tdim][gdim] % {"restriction": _choose_map[r]},
+    "generate min facet edge length": lambda tdim, gdim, r=None: min_facet_edge_length[tdim][gdim] % {"restriction": _choose_map(r)},
+    "generate max facet edge length": lambda tdim, gdim, r=None: max_facet_edge_length[tdim][gdim] % {"restriction": _choose_map(r)},
     "generate ip coordinates":  lambda g, num_ip, name, ip, r=None: (ip_coordinates[g][0], ip_coordinates[g][1] % \
-                                {"restriction": _choose_map[r], "ip": ip, "name": name, "num_ip": num_ip}),
-    "scale factor snippet":    scale_factor,
-    "map onto physical":       map_onto_physical,
-    "combinations":            combinations_snippet,
-    "transform snippet":       transform_snippet,
-    "evaluate function":       evaluate_f,
-    "ufc comment":             comment_ufc,
-    "dolfin comment":          comment_dolfin,
-    "header_h":                header_h,
-    "header_c":                header_c,
-    "footer":                  footer
-})
+                                {"restriction": _choose_map(r), "ip": ip, "name": name, "num_ip": num_ip}),
+    "scale factor snippet":     scale_factor,
+    "map onto physical":        map_onto_physical,
+    "evaluate basis snippet":   eval_basis,
+    "combinations":             combinations_snippet,
+    "transform snippet":        transform_snippet,
+    "evaluate function":        evaluate_f,
+    "ufc comment":              comment_ufc,
+    "dolfin comment":           comment_dolfin,
+    "header_h":                 header_h,
+    "header_c":                 header_c,
+    "footer":                   footer,
+    "eval_basis_decl":          eval_basis_decl,
+    "eval_basis":               eval_basis,
+    "eval_basis_copy":          eval_basis_copy,
+    "eval_derivs_decl":         eval_derivs_decl,
+    "eval_derivs":              eval_derivs,
+    "eval_derivs_copy":         eval_derivs_copy,
+    "extract_cell_coordinates": lambda offset, r : "const double* vertex_coordinates_%d = vertex_coordinates + %d;" % (r, offset)
+    })
 
 # Class names
 format.update({
@@ -290,6 +306,9 @@ format.update({
     "classname point_integral":  lambda prefix, form_id, sub_domain:\
               "%s_point_integral_%d_%s" % (prefix.lower(), form_id, sub_domain),
 
+    "classname custom_integral":  lambda prefix, form_id, sub_domain:\
+              "%s_custom_integral_%d_%s" % (prefix.lower(), form_id, sub_domain),
+
     "classname form": lambda prefix, i: "%s_form_%d" % (prefix.lower(), i)
 })
 
@@ -415,7 +434,7 @@ def _inner_product(v, w):
     return result
 
 def _transform(type, i, j, m, n, r):
-    map_name = {"J": "J", "JINV": "K"}[type] + _choose_map[r]
+    map_name = {"J": "J", "JINV": "K"}[type] + _choose_map(r)
     return (map_name + "[%d]") % _flatten(i, j, m, n)
 
 # FIXME: Input to _generate_switch should be a list of tuples (i, case)
@@ -510,7 +529,7 @@ def _generate_loop(lines, loop_vars, _indent):
     indices.reverse()
     for index in indices:
         _indent -= 2
-        code.append(indent(f_end + f_comment("end loop over '%s'" % index), _indent))
+        code.append(indent(f_end + " " + f_comment("end loop over '%s'" % index), _indent))
 
     return code
 
@@ -525,7 +544,7 @@ def _matrix_index(i, j, range_j):
         access = format["add"]([irj, j])
     return access
 
-def _generate_psi_name(counter, entitytype, entity, component, derivatives, avg):
+def _generate_psi_name(counter, entity_type, entity, component, derivatives, avg):
     """Generate a name for the psi table of the form:
     FE#_f#_v#_C#_D###_A#, where '#' will be an integer value.
 
@@ -546,9 +565,12 @@ def _generate_psi_name(counter, entitytype, entity, component, derivatives, avg)
 
     name = "FE%d" % counter
 
-    if entitytype == "facet":
-        name += "_f%d" % entity
-    elif entitytype == "vertex":
+    if entity_type == "facet":
+        if entity is None:
+            name += "_f0"
+        else:
+            name += "_f%d" % entity
+    elif entity_type == "vertex":
         name += "_v%d" % entity
 
     if component != () and component != []:
@@ -564,7 +586,7 @@ def _generate_psi_name(counter, entitytype, entity, component, derivatives, avg)
 
     return name
 
-def _generate_normal(tdim, gdim, domain_type, reference_normal=False):
+def _generate_normal(tdim, gdim, integral_type, reference_normal=False):
     "Generate code for computing normal"
 
     # Choose snippets
@@ -575,47 +597,51 @@ def _generate_normal(tdim, gdim, domain_type, reference_normal=False):
     normal = facet_normal[tdim][gdim]
 
     # Choose restrictions
-    if domain_type == "exterior_facet":
+    if integral_type == "exterior_facet":
         code = direction % {"restriction": "", "facet" : "facet"}
         code += normal % {"direction" : "", "restriction": ""}
-    elif domain_type == "interior_facet":
-        code = direction % {"restriction": _choose_map["+"], "facet": "facet_0"}
-        code += normal % {"direction" : "", "restriction": _choose_map["+"]}
-        code += normal % {"direction" : "!", "restriction": _choose_map["-"]}
+    elif integral_type == "interior_facet":
+        code = direction % {"restriction": _choose_map("+"), "facet": "facet_0"}
+        code += normal % {"direction" : "", "restriction": _choose_map("+")}
+        code += normal % {"direction" : "!", "restriction": _choose_map("-")}
     else:
-        error("Unsupported domain_type: %s" % str(domain_type))
+        error("Unsupported integral_type: %s" % str(integral_type))
     return code
 
-def _generate_cell_volume(tdim, gdim, domain_type):
+def _generate_cell_volume(tdim, gdim, integral_type, r=None):
     "Generate code for computing cell volume."
 
     # Choose snippets
     volume = cell_volume[tdim][gdim]
 
     # Choose restrictions
-    if domain_type in ("cell", "exterior_facet"):
+    if integral_type in ("cell", "exterior_facet"):
         code = volume % {"restriction": ""}
-    elif domain_type == "interior_facet":
-        code = volume % {"restriction": _choose_map["+"]}
-        code += volume % {"restriction": _choose_map["-"]}
+    elif integral_type == "interior_facet":
+        code = volume % {"restriction": _choose_map("+")}
+        code += volume % {"restriction": _choose_map("-")}
+    elif integral_type == "custom":
+        code = volume % {"restriction": _choose_map(r)}
     else:
-        error("Unsupported domain_type: %s" % str(domain_type))
+        error("Unsupported integral_type: %s" % str(integral_type))
     return code
 
-def _generate_circumradius(tdim, gdim, domain_type):
+def _generate_circumradius(tdim, gdim, integral_type, r=None):
     "Generate code for computing a cell's circumradius."
 
     # Choose snippets
     radius = circumradius[tdim][gdim]
 
     # Choose restrictions
-    if domain_type in ("cell", "exterior_facet", "point"):
+    if integral_type in ("cell", "exterior_facet", "point"):
         code = radius % {"restriction": ""}
-    elif domain_type == "interior_facet":
-        code = radius % {"restriction": _choose_map["+"]}
-        code += radius % {"restriction": _choose_map["-"]}
+    elif integral_type == "interior_facet":
+        code = radius % {"restriction": _choose_map("+")}
+        code += radius % {"restriction": _choose_map("-")}
+    elif integral_type == "custom":
+        code = radius % {"restriction": _choose_map(r)}
     else:
-        error("Unsupported domain_type: %s" % str(domain_type))
+        error("Unsupported integral_type: %s" % str(integral_type))
     return code
 
 def _flatten(i, j, m, n):
