@@ -40,18 +40,19 @@ except:
     _soya_imported = False
 
 # Colors for elements
-element_colors = {"Argyris":                  (0.45, 0.70, 0.80),
-                  "Arnold-Winther":           (0.00, 0.00, 1.00),
-                  "Brezzi-Douglas-Marini":    (1.00, 1.00, 0.00),
-                  "Crouzeix-Raviart":         (1.00, 0.25, 0.25),
-                  "Discontinuous Lagrange":   (0.00, 0.25, 0.00),
-                  "Hermite":                  (0.50, 1.00, 0.50),
-                  "Lagrange":                 (0.00, 1.00, 0.00),
-                  "Mardal-Tai-Winther":       (1.00, 0.10, 0.90),
-                  "Morley":                   (0.40, 0.40, 0.40),
-                  "Nedelec 1st kind H(curl)": (0.90, 0.30, 0.00),
-                  "Nedelec 2nd kind H(curl)": (0.70, 0.20, 0.00),
-                  "Raviart-Thomas":           (0.90, 0.60, 0.00)}
+element_colors = {"Argyris":                      (0.45, 0.70, 0.80),
+                  "Arnold-Winther":               (0.00, 0.00, 1.00),
+                  "Brezzi-Douglas-Marini":        (1.00, 1.00, 0.00),
+                  "Crouzeix-Raviart":             (1.00, 0.25, 0.25),
+                  "Discontinuous Lagrange":       (0.00, 0.25, 0.00),
+                  "Discontinuous Raviart-Thomas": (0.90, 0.90, 0.30),
+                  "Hermite":                      (0.50, 1.00, 0.50),
+                  "Lagrange":                     (0.00, 1.00, 0.00),
+                  "Mardal-Tai-Winther":           (1.00, 0.10, 0.90),
+                  "Morley":                       (0.40, 0.40, 0.40),
+                  "Nedelec 1st kind H(curl)":     (0.90, 0.30, 0.00),
+                  "Nedelec 2nd kind H(curl)":     (0.70, 0.20, 0.00),
+                  "Raviart-Thomas":               (0.90, 0.60, 0.00)}
 
 def plot(element, rotate=True):
     "Plot finite element."
@@ -75,14 +76,16 @@ def plot(element, rotate=True):
         # Create cell model
         cell, is3d = create_cell_model(element)
 
+        cellname = element.cell().cellname() # Assuming single cell
+
         # Create dof models
         dofs, num_moments = create_dof_models(element)
 
         # Create title
         if element.degree() is not None:
-            title = "%s of degree %d on a %s" % (element.family(), element.degree(), element.cell().cellname())
+            title = "%s of degree %d on a %s" % (element.family(), element.degree(), cellname)
         else:
-            title = "%s on a %s" % (element.family(), element.cell().cellname())
+            title = "%s on a %s" % (element.family(), cellname)
 
         # Render plot window
         render([cell] + dofs, title, num_moments, is3d, rotate)
