@@ -20,7 +20,7 @@
 # Modified by Anders Logg, 2009
 #
 # First added:  2009-03-18
-# Last changed: 2011-11-22
+# Last changed: 2014-04-23
 
 # Python modules.
 from numpy import shape
@@ -270,14 +270,52 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
     # -------------------------------------------------------------------------
     # FacetNormal, CellVolume, Circumradius, FacetArea (geometry.py).
     # -------------------------------------------------------------------------
-    def facet_normal(self, o,  *operands):
-        #print("Visiting FacetNormal:")
+    def cell_coordinate(self, o): # FIXME
+        error("This object should be implemented by the child class.")
 
-        # Get the component
+    def facet_coordinate(self, o): # FIXME
+        error("This object should be implemented by the child class.")
+
+    def cell_origin(self, o): # FIXME
+        error("This object should be implemented by the child class.")
+
+    def facet_origin(self, o): # FIXME
+        error("This object should be implemented by the child class.")
+
+    def cell_facet_origin(self, o): # FIXME
+        error("This object should be implemented by the child class.")
+
+    def jacobian(self, o): # FIXME
+        error("This object should be implemented by the child class.")
+
+    def jacobian_determinant(self, o): # FIXME
+        error("This object should be implemented by the child class.")
+
+    def jacobian_inverse(self, o): # FIXME
+        error("This object should be implemented by the child class.")
+
+    def facet_jacobian(self, o): # FIXME
+        error("This object should be implemented by the child class.")
+
+    def facet_jacobian_determinant(self, o): # FIXME
+        error("This object should be implemented by the child class.")
+
+    def facet_jacobian_inverse(self, o): # FIXME
+        error("This object should be implemented by the child class.")
+
+    def cell_facet_jacobian(self, o): # FIXME
+        error("This object should be implemented by the child class.")
+
+    def cell_facet_jacobian_determinant(self, o): # FIXME
+        error("This object should be implemented by the child class.")
+
+    def cell_facet_jacobian_inverse(self, o): # FIXME
+        error("This object should be implemented by the child class.")
+
+    def facet_normal(self, o):
         components = self.component()
 
         # Safety check.
-        ffc_assert(not operands, "Didn't expect any operands for FacetNormal: " + repr(operands))
         ffc_assert(len(components) == 1,
                    "FacetNormal expects 1 component index: " + repr(components))
 
@@ -291,24 +329,21 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
 
         return {(): create_symbol(normal_component, GEO)}
 
-    def cell_volume(self, o,  *operands):
-        # Safety check.
-        ffc_assert(not operands, "Didn't expect any operands for CellVolume: " + repr(operands))
+    def cell_normal(self, o): # FIXME
+        error("This object should be implemented by the child class.")
 
+    def cell_volume(self, o):
         # FIXME: KBO: This has to change for higher order elements
-#        detJ = format["det(J)"](self.restriction)
-#        volume = format["absolute value"](detJ)
-#        self.trans_set.add(detJ)
+        #detJ = format["det(J)"](self.restriction)
+        #volume = format["absolute value"](detJ)
+        #self.trans_set.add(detJ)
 
         volume = format["cell volume"](self.restriction)
         self.trans_set.add(volume)
 
         return {():create_symbol(volume, GEO)}
 
-    def circumradius(self, o,  *operands):
-        # Safety check.
-        ffc_assert(not operands, "Didn't expect any operands for Circumradius: " + repr(operands))
-
+    def circumradius(self, o):
         # FIXME: KBO: This has to change for higher order elements
         circumradius = format["circumradius"](self.restriction)
         self.trans_set.add(circumradius)
@@ -316,7 +351,6 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
         return {():create_symbol(circumradius, GEO)}
 
     def facet_area(self, o):
-
         # FIXME: KBO: This has to change for higher order elements
         # NOTE: Omitting restriction because the area of a facet is the same
         # on both sides.
@@ -330,7 +364,8 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
     def min_facet_edge_length(self, o):
         # FIXME: this has no meaning for cell integrals. (Need check in FFC or UFL).
 
-        if self.tdim < 3:
+        tdim = self.tdim # FIXME: o.domain().topological_dimension() ???
+        if tdim < 3:
             return self.facet_area(o)
 
         edgelen = format["min facet edge length"](self.restriction)
@@ -341,7 +376,8 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
     def max_facet_edge_length(self, o):
         # FIXME: this has no meaning for cell integrals. (Need check in FFC or UFL).
 
-        if self.tdim < 3:
+        tdim = self.tdim # FIXME: o.domain().topological_dimension() ???
+        if tdim < 3:
             return self.facet_area(o)
 
         edgelen = format["max facet edge length"](self.restriction)
@@ -349,13 +385,18 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
 
         return {():create_symbol(edgelen, GEO)}
 
+    def cell_orientation(self, o): # FIXME
+        error("This object should be implemented by the child class.")
+
+    def quadrature_weight(self, o): # FIXME
+        error("This object should be implemented by the child class.")
+
     # -------------------------------------------------------------------------
 
     def create_argument(self, ufl_argument, derivatives, component, local_comp,
                         local_offset, ffc_element, transformation, multiindices,
                         tdim, gdim, avg):
         "Create code for basis functions, and update relevant tables of used basis."
-        ffc_assert(ufl_argument in self._function_replace_values, "Expecting ufl_argument to have been mapped prior to this call.")
 
         # Prefetch formats to speed up code generation.
         f_transform     = format["transform"]
@@ -368,7 +409,7 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
         if transformation == "affine":
             # Loop derivatives and get multi indices.
             for multi in multiindices:
-                deriv = [multi.count(i) for i in range(self.tdim)]
+                deriv = [multi.count(i) for i in range(tdim)]
                 if not any(deriv):
                     deriv = []
 
@@ -387,11 +428,11 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
 
             # Loop derivatives and get multi indices.
             for multi in multiindices:
-                deriv = [multi.count(i) for i in range(self.tdim)]
+                deriv = [multi.count(i) for i in range(tdim)]
                 if not any(deriv):
                     deriv = []
 
-                for c in range(self.tdim):
+                for c in range(tdim):
                     # Create mapping and basis name.
                     mapping, basis = self._create_mapping_basis(c + local_offset, deriv, avg, ufl_argument, ffc_element)
                     if not mapping in code:
@@ -438,7 +479,7 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
         if transformation == "affine":
             # Loop derivatives and get multi indices.
             for multi in multiindices:
-                deriv = [multi.count(i) for i in range(self.tdim)]
+                deriv = [multi.count(i) for i in range(tdim)]
                 if not any(deriv):
                     deriv = []
 
@@ -454,11 +495,11 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
 
             # Loop derivatives and get multi indices.
             for multi in multiindices:
-                deriv = [multi.count(i) for i in range(self.tdim)]
+                deriv = [multi.count(i) for i in range(tdim)]
                 if not any(deriv):
                     deriv = []
 
-                for c in range(self.tdim):
+                for c in range(tdim):
                     function_name = self._create_function_name(c + local_offset, deriv, avg, is_quad_element, ufl_function, ffc_element)
                     if function_name:
                         # Multiply basis by appropriate transform.
@@ -493,10 +534,12 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
 
         # Add transformation if needed.
         transforms = []
-        for i, direction in enumerate(derivatives):
-            ref = multi[i]
-            t = f_transform("JINV", ref, direction, tdim, gdim, self.restriction)
-            transforms.append(create_symbol(t, GEO))
+        if not self.integral_type == "custom":
+            for i, direction in enumerate(derivatives):
+                ref = multi[i]
+                t = f_transform("JINV", ref, direction, tdim, gdim, self.restriction)
+                transforms.append(create_symbol(t, GEO))
+
         transforms.append(function)
         return create_product(transforms)
 
@@ -554,7 +597,7 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
     def _count_operations(self, expression):
         return expression.ops()
 
-    def _create_entry_data(self, val, domain_type):
+    def _create_entry_data(self, val, integral_type):
 #        zero = False
         # Multiply value by weight and determinant
         ACCESS = GEO
@@ -565,7 +608,7 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
         weight = self._create_symbol(weight, ACCESS)[()]
 
         # Create value.
-        if domain_type == "point":
+        if integral_type in ("point", "custom"):
             trans_set = set()
             value = create_product([val, weight])
         else:
