@@ -19,10 +19,10 @@
 # Modified by Martin Alnaes, 2013
 #
 # First added:  2010-02-08
-# Last changed: 2013-02-10
+# Last changed: 2014-04-23
 
 # FFC modules
-from ffc.log import info, error
+from ffc.log import info, error, warning
 from ffc.cpp import format
 from ffc.quadrature.symbolics import optimise_code, BASIS, IP, GEO, CONST
 from ffc.quadrature.symbolics import create_product, create_sum, create_symbol, create_fraction
@@ -33,28 +33,36 @@ def optimize_integral_ir(ir, parameters):
     # FIXME: input argument "parameters" has been added to optimize_integral_ir
     # FIXME: which shadows a local parameter
 
+    # Get integral type and optimization parameters
+    integral_type = ir["integral_type"]
     parameters = ir["optimise_parameters"]
+
+    # Check whether we should optimize
     if parameters["optimisation"]:
-        integrals  =  ir["trans_integrals"]
-        domain_type = ir["domain_type"]
-        num_facets =  ir["num_facets"]
-        num_vertices =  ir["num_vertices"]
-        geo_consts =  ir["geo_consts"]
-        psi_tables_map =  ir["psi_tables_map"]
-        if domain_type == "cell":
+
+        # Get parameters
+        integrals      = ir["trans_integrals"]
+        integral_type  = ir["integral_type"]
+        num_facets     = ir["num_facets"]
+        num_vertices   = ir["num_vertices"]
+        geo_consts     = ir["geo_consts"]
+        psi_tables_map = ir["psi_tables_map"]
+
+        # Optimize based on integral type
+        if integral_type == "cell":
             info("Optimising expressions for cell integral")
             if parameters["optimisation"] in ("precompute_ip_const", "precompute_basis_const"):
                 _precompute_expressions(integrals, geo_consts, parameters["optimisation"])
             else:
                 _simplify_expression(integrals, geo_consts, psi_tables_map)
-        elif domain_type == "exterior_facet":
+        elif integral_type == "exterior_facet":
             for i in range(num_facets):
                 info("Optimising expressions for facet integral %d" % i)
                 if parameters["optimisation"] in ("precompute_ip_const", "precompute_basis_const"):
                     _precompute_expressions(integrals[i], geo_consts, parameters["optimisation"])
                 else:
                     _simplify_expression(integrals[i], geo_consts, psi_tables_map)
-        elif domain_type == "interior_facet":
+        elif integral_type == "interior_facet":
             for i in range(num_facets):
                 for j in range(num_facets):
                     info("Optimising expressions for facet integral (%d, %d)" % (i, j))
@@ -62,7 +70,7 @@ def optimize_integral_ir(ir, parameters):
                         _precompute_expressions(integrals[i][j], geo_consts,parameters["optimisation"])
                     else:
                         _simplify_expression(integrals[i][j], geo_consts, psi_tables_map)
-        elif domain_type == "point":
+        elif integral_type == "point":
             for i in range(num_vertices):
                 info("Optimising expressions for poin integral %d" % i)
                 if parameters["optimisation"] in ("precompute_ip_const", "precompute_basis_const"):
@@ -70,7 +78,7 @@ def optimize_integral_ir(ir, parameters):
                 else:
                     _simplify_expression(integrals[i], geo_consts, psi_tables_map)
         else:
-            error("Unhandled domain type: " + str(domain_type))
+            error("Unhandled domain type: " + str(integral_type))
 
     return ir
 
