@@ -21,7 +21,7 @@
 # Modified by Anders Logg 2009, 2013
 #
 # First added:  2009-02-09
-# Last changed: 2013-01-10
+# Last changed: 2014-04-23
 
 # Python modules.
 from numpy import shape
@@ -351,6 +351,48 @@ class QuadratureTransformer(QuadratureTransformerBase):
     # -------------------------------------------------------------------------
     # FacetNormal, CellVolume, Circumradius, FacetArea (geometry.py).
     # -------------------------------------------------------------------------
+    def cell_coordinate(self, o): # FIXME
+        error("This object should be implemented by the child class.")
+
+    def facet_coordinate(self, o): # FIXME
+        error("This object should be implemented by the child class.")
+
+    def cell_origin(self, o): # FIXME
+        error("This object should be implemented by the child class.")
+
+    def facet_origin(self, o): # FIXME
+        error("This object should be implemented by the child class.")
+
+    def cell_facet_origin(self, o): # FIXME
+        error("This object should be implemented by the child class.")
+
+    def jacobian(self, o): # FIXME
+        error("This object should be implemented by the child class.")
+
+    def jacobian_determinant(self, o): # FIXME
+        error("This object should be implemented by the child class.")
+
+    def jacobian_inverse(self, o): # FIXME
+        error("This object should be implemented by the child class.")
+
+    def facet_jacobian(self, o): # FIXME
+        error("This object should be implemented by the child class.")
+
+    def facet_jacobian_determinant(self, o): # FIXME
+        error("This object should be implemented by the child class.")
+
+    def facet_jacobian_inverse(self, o): # FIXME
+        error("This object should be implemented by the child class.")
+
+    def cell_facet_jacobian(self, o): # FIXME
+        error("This object should be implemented by the child class.")
+
+    def cell_facet_jacobian_determinant(self, o): # FIXME
+        error("This object should be implemented by the child class.")
+
+    def cell_facet_jacobian_inverse(self, o): # FIXME
+        error("This object should be implemented by the child class.")
+
     def facet_normal(self, o):
         #print("Visiting FacetNormal:")
 
@@ -370,6 +412,9 @@ class QuadratureTransformer(QuadratureTransformerBase):
 
         return {():normal_component}
 
+    def cell_normal(self, o): # FIXME
+        error("This object should be implemented by the child class.")
+
     def cell_volume(self, o):
         # FIXME: KBO: This has to change for higher order elements
         volume = format["cell volume"](self.restriction)
@@ -398,7 +443,8 @@ class QuadratureTransformer(QuadratureTransformerBase):
     def min_facet_edge_length(self, o):
         # FIXME: this has no meaning for cell integrals. (Need check in FFC or UFL).
 
-        if self.tdim < 3:
+        tdim = self.tdim # FIXME: o.domain().topological_dimension()
+        if tdim < 3:
             return self.facet_area(o)
 
         edgelen = format["min facet edge length"](self.restriction)
@@ -409,7 +455,8 @@ class QuadratureTransformer(QuadratureTransformerBase):
     def max_facet_edge_length(self, o):
         # FIXME: this has no meaning for cell integrals. (Need check in FFC or UFL).
 
-        if self.tdim < 3:
+        tdim = self.tdim # FIXME: o.domain().topological_dimension()
+        if tdim < 3:
             return self.facet_area(o)
 
         edgelen = format["max facet edge length"](self.restriction)
@@ -417,13 +464,18 @@ class QuadratureTransformer(QuadratureTransformerBase):
 
         return {():edgelen}
 
+    def cell_orientation(self, o): # FIXME
+        error("This object should be implemented by the child class.")
+
+    def quadrature_weight(self, o): # FIXME
+        error("This object should be implemented by the child class.")
+
     # -------------------------------------------------------------------------
 
     def create_argument(self, ufl_argument, derivatives, component, local_comp,
                         local_offset, ffc_element, transformation, multiindices,
                         tdim, gdim, avg):
         "Create code for basis functions, and update relevant tables of used basis."
-        ffc_assert(ufl_argument in self._function_replace_values, "Expecting ufl_argument to have been mapped prior to this call.")
 
         # Prefetch formats to speed up code generation.
         f_group         = format["grouping"]
@@ -438,9 +490,10 @@ class QuadratureTransformer(QuadratureTransformerBase):
 
         # Handle affine mappings.
         if transformation == "affine":
+
             # Loop derivatives and get multi indices.
             for multi in multiindices:
-                deriv = [multi.count(i) for i in range(self.tdim)]
+                deriv = [multi.count(i) for i in range(tdim)]
                 if not any(deriv):
                     deriv = []
 
@@ -460,11 +513,11 @@ class QuadratureTransformer(QuadratureTransformerBase):
 
             # Loop derivatives and get multi indices.
             for multi in multiindices:
-                deriv = [multi.count(i) for i in range(self.tdim)]
+                deriv = [multi.count(i) for i in range(tdim)]
                 if not any(deriv):
                     deriv = []
 
-                for c in range(self.tdim):
+                for c in range(tdim):
                     # Create mapping and basis name.
                     mapping, basis = self._create_mapping_basis(c + local_offset, deriv, avg, ufl_argument, ffc_element)
                     if not mapping in code:
@@ -519,7 +572,7 @@ class QuadratureTransformer(QuadratureTransformerBase):
         if transformation == "affine":
             # Loop derivatives and get multi indices.
             for multi in multiindices:
-                deriv = [multi.count(i) for i in range(self.tdim)]
+                deriv = [multi.count(i) for i in range(tdim)]
                 if not any(deriv):
                     deriv = []
 
@@ -535,11 +588,11 @@ class QuadratureTransformer(QuadratureTransformerBase):
 
             # Loop derivatives and get multi indices.
             for multi in multiindices:
-                deriv = [multi.count(i) for i in range(self.tdim)]
+                deriv = [multi.count(i) for i in range(tdim)]
                 if not any(deriv):
                     deriv = []
 
-                for c in range(self.tdim):
+                for c in range(tdim):
                     function_name = self._create_function_name(c + local_offset, deriv, avg, is_quad_element, ufl_function, ffc_element)
                     if function_name:
                         # Multiply basis by appropriate transform.
@@ -578,11 +631,12 @@ class QuadratureTransformer(QuadratureTransformerBase):
 
         # Add transformation if needed.
         transforms = []
-        for i, direction in enumerate(derivatives):
-            ref = multi[i]
-            t = f_transform("JINV", ref, direction, tdim, gdim, self.restriction)
-            self.trans_set.add(t)
-            transforms.append(t)
+        if not self.integral_type == "custom":
+            for i, direction in enumerate(derivatives):
+                ref = multi[i]
+                t = f_transform("JINV", ref, direction, tdim, gdim, self.restriction)
+                self.trans_set.add(t)
+                transforms.append(t)
 
         # Only multiply by basis if it is present.
         if function:
@@ -622,6 +676,11 @@ class QuadratureTransformer(QuadratureTransformerBase):
     def _atan_2_function(self, operands, format_function):
         x1,x2 = operands
         x1,x2 = x1.values()[0],x2.values()[0]
+
+        if x1 is None:
+            x1 = format["floating point"](0.0)
+        if x2 is None:
+            x2 = format["floating point"](0.0)
         return {():format_function(x1, x2)}
 
     def _bessel_function(self, operands, format_function):
@@ -652,15 +711,16 @@ class QuadratureTransformer(QuadratureTransformerBase):
     def _count_operations(self, expression):
         return operation_count(expression, format)
 
-    def _create_entry_data(self, val, domain_type):
+    def _create_entry_data(self, val, integral_type):
         # Multiply value by weight and determinant
         # Create weight and scale factor.
+
         weight = format["weight"](self.points)
-        if self.points > 1:
+        if self.points is None or self.points > 1:
             weight += format["component"]("", format["integration points"])
 
         # Update sets of used variables.
-        if domain_type == "point":
+        if integral_type in ("point", "custom"):
             trans_set = set()
             value = format["mul"]([val, weight])
         else:
