@@ -22,7 +22,7 @@ transformers to translate UFL expressions."""
 # Modified by Garth N. Wells, 2013
 #
 # First added:  2009-10-13
-# Last changed: 2013-02-20
+# Last changed: 2014-04-23
 
 # Python modules.
 from itertools import izip
@@ -58,7 +58,7 @@ class QuadratureTransformerBase(Transformer):
                  quad_weights,
                  gdim,
                  tdim,
-                 entitytype,
+                 entity_type,
                  function_replace_map,
                  optimise_parameters):
 
@@ -83,7 +83,7 @@ class QuadratureTransformerBase(Transformer):
         self.function_data = {}
         self.tdim = tdim
         self.gdim = gdim
-        self.entitytype = entitytype
+        self.entity_type = entity_type
         self.points = 0
         self.facet0 = None
         self.facet1 = None
@@ -101,14 +101,14 @@ class QuadratureTransformerBase(Transformer):
         self._components = Stack()
 
         self.element_map, self.name_map, self.unique_tables =\
-            create_psi_tables(psi_tables, self.optimise_parameters["eliminate zeros"], self.entitytype)
+            create_psi_tables(psi_tables, self.optimise_parameters["eliminate zeros"], self.entity_type)
 
         # Cache.
         self.argument_cache = {}
         self.function_cache = {}
 
     def update_cell(self):
-        ffc_assert(self.entitytype == "cell", "Not expecting update_cell on a %s." % self.entitytype)
+        ffc_assert(self.entity_type == "cell", "Not expecting update_cell on a %s." % self.entity_type)
         self.facet0 = None
         self.facet1 = None
         self.vertex = None
@@ -116,7 +116,7 @@ class QuadratureTransformerBase(Transformer):
         self.conditionals = {}
 
     def update_facets(self, facet0, facet1):
-        ffc_assert(self.entitytype == "facet", "Not expecting update_facet on a %s." % self.entitytype)
+        ffc_assert(self.entity_type == "facet", "Not expecting update_facet on a %s." % self.entity_type)
         self.facet0 = facet0
         self.facet1 = facet1
         self.vertex = None
@@ -124,7 +124,7 @@ class QuadratureTransformerBase(Transformer):
         self.conditionals = {}
 
     def update_vertex(self, vertex):
-        ffc_assert(self.entitytype == "vertex", "Not expecting update_vertex on a %s." % self.entitytype)
+        ffc_assert(self.entity_type == "vertex", "Not expecting update_vertex on a %s." % self.entity_type)
         self.facet0 = None
         self.facet1 = None
         self.vertex = vertex
@@ -267,16 +267,73 @@ class QuadratureTransformerBase(Transformer):
     # -------------------------------------------------------------------------
     # FacetNormal, CellVolume, Circumradius (geometry.py).
     # -------------------------------------------------------------------------
+    def cell_coordinate(self, o):
+        error("This object should be implemented by the child class.")
+
+    def facet_coordinate(self, o):
+        error("This object should be implemented by the child class.")
+
+    def cell_origin(self, o):
+        error("This object should be implemented by the child class.")
+
+    def facet_origin(self, o):
+        error("This object should be implemented by the child class.")
+
+    def cell_facet_origin(self, o):
+        error("This object should be implemented by the child class.")
+
+    def jacobian(self, o):
+        error("This object should be implemented by the child class.")
+
+    def jacobian_determinant(self, o):
+        error("This object should be implemented by the child class.")
+
+    def jacobian_inverse(self, o):
+        error("This object should be implemented by the child class.")
+
+    def facet_jacobian(self, o):
+        error("This object should be implemented by the child class.")
+
+    def facet_jacobian_determinant(self, o):
+        error("This object should be implemented by the child class.")
+
+    def facet_jacobian_inverse(self, o):
+        error("This object should be implemented by the child class.")
+
+    def cell_facet_jacobian(self, o):
+        error("This object should be implemented by the child class.")
+
+    def cell_facet_jacobian_determinant(self, o):
+        error("This object should be implemented by the child class.")
+
+    def cell_facet_jacobian_inverse(self, o):
+        error("This object should be implemented by the child class.")
+
     def facet_normal(self, o):
-        print "\n\nVisiting FacetNormal: ", repr(o)
+        error("This object should be implemented by the child class.")
+
+    def cell_normal(self, o):
         error("This object should be implemented by the child class.")
 
     def cell_volume(self, o):
-        print "\n\nVisiting CellVolume: ", repr(o)
         error("This object should be implemented by the child class.")
 
     def circumradius(self, o):
-        print "\n\nVisiting Circumeradius: ", repr(o)
+        error("This object should be implemented by the child class.")
+
+    def facet_area(self, o):
+        error("This object should be implemented by the child class.")
+
+    def min_facet_edge_length(self, o):
+        error("This object should be implemented by the child class.")
+
+    def max_facet_edge_length(self, o):
+        error("This object should be implemented by the child class.")
+
+    def cell_orientation(self, o):
+        error("This object should be implemented by the child class.")
+
+    def quadrature_weight(self, o):
         error("This object should be implemented by the child class.")
 
     # -------------------------------------------------------------------------
@@ -288,9 +345,6 @@ class QuadratureTransformerBase(Transformer):
     def argument(self, o):
         #print("\nVisiting Argument:" + repr(o))
 
-        # Map o to object with proper element and numbering
-        o = self._function_replace_map[o]
-
         # Create aux. info.
         components = self.component()
         derivatives = self.derivatives()
@@ -299,6 +353,8 @@ class QuadratureTransformerBase(Transformer):
         key = (o, components, derivatives, self.restriction, self.avg)
         basis = self.argument_cache.get(key, None)
 
+        tdim = self.tdim # FIXME: o.domain().topological_dimension() ???
+
         # FIXME: Why does using a code dict from cache make the expression manipulations blow (MemoryError) up later?
         if basis is None or self.optimise_parameters["optimisation"]:
             # Get auxiliary variables to generate basis
@@ -309,7 +365,7 @@ class QuadratureTransformerBase(Transformer):
             basis = self.create_argument(o, derivatives, component, local_comp,
                                          local_offset, ffc_element,
                                          transformation, multiindices,
-                                         self.tdim, self.gdim, self.avg)
+                                         tdim, self.gdim, self.avg)
             self.argument_cache[key] = basis
 
         return basis
@@ -382,7 +438,7 @@ class QuadratureTransformerBase(Transformer):
     def coefficient(self, o):
         #print("\nVisiting Coefficient: " + repr(o))
 
-        # Map o to object with proper element and numbering
+        # Map o to object with proper element and count
         o = self._function_replace_map[o]
 
         # Create aux. info.
@@ -404,10 +460,12 @@ class QuadratureTransformerBase(Transformer):
             ffc_assert(not (derivatives and is_quad_element), \
                        "Derivatives of Quadrature elements are not supported: " + repr(o))
 
+            tdim = self.tdim # FIXME: o.domain().topological_dimension() ???
+
             # Create code for function and add empty tuple to cache dict.
             function_code = {(): self.create_function(o, derivatives, component,
                                                       local_comp, local_offset, ffc_element, is_quad_element,
-                                                      transformation, multiindices, self.tdim, self.gdim, self.avg)}
+                                                      transformation, multiindices, tdim, self.gdim, self.avg)}
 
             self.function_cache[key] = function_code
 
@@ -416,7 +474,7 @@ class QuadratureTransformerBase(Transformer):
     def constant(self, o):
         #print("\n\nVisiting Constant: " + repr(o))
 
-        # Map o to object with proper element and numbering
+        # Map o to object with proper element and count
         o = self._function_replace_map[o]
 
         # Safety checks.
@@ -437,7 +495,7 @@ class QuadratureTransformerBase(Transformer):
     def vector_constant(self, o):
         #print("\n\nVisiting VectorConstant: " + repr(o))
 
-        # Map o to object with proper element and numbering
+        # Map o to object with proper element and count
         o = self._function_replace_map[o]
 
         # Get the component
@@ -460,7 +518,7 @@ class QuadratureTransformerBase(Transformer):
     def tensor_constant(self, o):
         #print("\n\nVisiting TensorConstant: " + repr(o))
 
-        # Map o to object with proper element and numbering
+        # Map o to object with proper element and count
         o = self._function_replace_map[o]
 
         # Get the components
@@ -689,7 +747,7 @@ class QuadratureTransformerBase(Transformer):
 
     def facet_avg(self, o):
         ffc_assert(self.avg is None, "Not expecting nested averages.")
-        ffc_assert(self.entitytype != "cell", "Cannot take facet_avg in a cell integral.")
+        ffc_assert(self.entity_type != "cell", "Cannot take facet_avg in a cell integral.")
 
         # Just get the first operand, there should only be one.
         expr, = o.operands()
@@ -766,8 +824,12 @@ class QuadratureTransformerBase(Transformer):
     # -------------------------------------------------------------------------
     # Generate terms for representation.
     # -------------------------------------------------------------------------
-    def generate_terms(self, integrand, domain_type):
+    def generate_terms(self, integrand, integral_type):
         "Generate terms for code generation."
+
+        # Set domain type
+        self.integral_type = integral_type
+
         #print integrand
         #print tree_format(integrand, 0, False)
         # Get terms.
@@ -783,7 +845,7 @@ class QuadratureTransformerBase(Transformer):
             if val is None:
                 continue
             # Create data.
-            value, ops, sets = self._create_entry_data(val, domain_type)
+            value, ops, sets = self._create_entry_data(val, integral_type)
             # Extract nzc columns if any and add to sets.
             used_nzcs = set([int(k[1].split(f_nzc)[1].split("[")[0]) for k in key if f_nzc in k[1]])
             sets.append(used_nzcs)
@@ -907,47 +969,45 @@ class QuadratureTransformerBase(Transformer):
         transformation = ffc_sub_element.mapping()[0]
 
         # Generate FFC multi index for derivatives.
-        multiindices = FFCMultiIndex([range(self.tdim)]*len(derivatives)).indices
+        tdim = self.tdim # FIXME: ufl_element.domain().topological_dimension() ???
+        multiindices = FFCMultiIndex([range(tdim)]*len(derivatives)).indices
 
         #print "in create_auxiliary"
         #print "component = ", component
         return (component, local_elem, local_comp, local_offset, ffc_element, transformation, multiindices)
 
     def _get_current_entity(self):
-        if self.entitytype == "cell":
+        if self.entity_type == "cell":
             # If we add macro cell integration, I guess the 'current cell number' would go here?
             return 0
-        elif self.entitytype == "facet":
+        elif self.entity_type == "facet":
             # Handle restriction through facet.
             return {"+": self.facet0, "-": self.facet1, None: self.facet0}[self.restriction]
-        elif self.entitytype == "vertex":
+        elif self.entity_type == "vertex":
             return self.vertex
         else:
-            error("Unknown entity type %s." % self.entitytype)
+            error("Unknown entity type %s." % self.entity_type)
 
     def _create_mapping_basis(self, component, deriv, avg, ufl_argument, ffc_element):
         "Create basis name and mapping from given basis_info."
-        ffc_assert(ufl_argument in self._function_replace_values, "Expecting ufl_argument to have been mapped prior to this call.")
 
         # Get string for integration points.
         f_ip = "0" if (avg or self.points == 1) else format["integration points"]
         generate_psi_name = format["psi name"]
 
         # Only support test and trial functions.
-        indices = { 0: format["first free index"],
-                    1: format["second free index"]}
+        indices = {0: format["first free index"],
+                   1: format["second free index"]}
 
         # Check that we have a basis function.
-        ffc_assert(ufl_argument.count() in indices, \
-                   "Currently, Argument index must be either 0 or 1: " + repr(ufl_argument))
+        ffc_assert(ufl_argument.number() in indices,
+                   "Currently, Argument number must be either 0 or 1: " + repr(ufl_argument))
+        ffc_assert(ufl_argument.part() is None,
+                   "Currently, Argument part is not supporte: " + repr(ufl_argument))
 
         # Get element counter and loop index.
         element_counter = self.element_map[1 if avg else self.points][ufl_argument.element()]
-        loop_index = indices[ufl_argument.count()]
-
-        # Create basis access, we never need to map the entry in the basis table
-        # since we will either loop the entire space dimension or the non-zeros.
-        basis_access = format["component"]("", [f_ip, loop_index])
+        loop_index = indices[ufl_argument.number()]
 
         # Offset element space dimension in case of negative restriction,
         # need to use the complete element for offset in case of mixed element.
@@ -955,13 +1015,21 @@ class QuadratureTransformerBase(Transformer):
         offset = {"+": "", "-": str(space_dim), None: ""}[self.restriction]
 
         # If we have a restricted function multiply space_dim by two.
-        if self.restriction == "+" or self.restriction == "-":
+        if self.restriction in ("+", "-"):
             space_dim *= 2
 
+        # If we have a restricted function and domain type is custom,
+        # then offset also the basis function access
+        if self.restriction in ("+", "-") and self.integral_type == "custom" and offset != "":
+            loop_index = format["add"]([loop_index, offset])
+
+        # Create basis access, we never need to map the entry in the basis table
+        # since we will either loop the entire space dimension or the non-zeros.
+        basis_access = format["component"]("", [f_ip, loop_index])
+
         # Get current cell entity, with current restriction considered
         entity = self._get_current_entity()
-
-        name = generate_psi_name(element_counter, self.entitytype, entity, component, deriv, avg)
+        name = generate_psi_name(element_counter, self.entity_type, entity, component, deriv, avg)
         name, non_zeros, zeros, ones = self.name_map[name]
         loop_index_range = shape(self.unique_tables[name])[1]
 
@@ -998,12 +1066,13 @@ class QuadratureTransformerBase(Transformer):
         # Example dx and ds: (0, j, 3, 3)
         # Example dS: (0, (j + 3), 3, 6), 6=2*space_dim
         # Example dS optimised: (0, (nz2[j] + 3), 2, 6), 6=2*space_dim
-        mapping = ((ufl_argument.count(), basis_map, loop_index_range, space_dim),)
+        mapping = ((ufl_argument.number(), basis_map, loop_index_range, space_dim),)
 
         return (mapping, basis)
 
     def _create_function_name(self, component, deriv, avg, is_quad_element, ufl_function, ffc_element):
-        ffc_assert(ufl_function in self._function_replace_values, "Expecting ufl_function to have been mapped prior to this call.")
+        ffc_assert(ufl_function in self._function_replace_values,
+                   "Expecting ufl_function to have been mapped prior to this call.")
 
         # Get string for integration points.
         f_ip = "0" if (avg or self.points == 1) else format["integration points"]
@@ -1019,7 +1088,7 @@ class QuadratureTransformerBase(Transformer):
 
         # Create basis name and map to correct basis and get info.
         generate_psi_name = format["psi name"]
-        psi_name = generate_psi_name(element_counter, self.entitytype, entity, component, deriv, avg)
+        psi_name = generate_psi_name(element_counter, self.entity_type, entity, component, deriv, avg)
         psi_name, non_zeros, zeros, ones = self.name_map[psi_name]
 
         # If all basis are zero we just return None.
