@@ -1,7 +1,7 @@
 """This module contains utility functions for some code shared between
 quadrature and tensor representation."""
 
-# Copyright (C) 2012-2013 Marie Rognes
+# Copyright (C) 2012-2014 Marie Rognes
 #
 # This file is part of FFC.
 #
@@ -18,14 +18,18 @@ quadrature and tensor representation."""
 # You should have received a copy of the GNU Lesser General Public License
 # along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
-# Modified by Martin Alnaes, 2013
+# Modified by Martin Alnaes 2013
+# Modified by Anders Logg 2014
 #
 # First added:  2013-01-08
-# Last changed: 2013-02-10
+# Last changed: 2014-04-15
+
+from ufl.measure import integral_type_to_measure_name
 
 from ffc.fiatinterface import create_element
 from ffc.fiatinterface import cellname_to_num_entities
 from ffc.cpp import format
+from ffc.log import error
 
 def transform_component(component, offset, ufl_element):
     """
@@ -39,8 +43,9 @@ def transform_component(component, offset, ufl_element):
     # This code is used for tensor/monomialtransformation.py and
     # quadrature/quadraturetransformerbase.py.
 
-    gdim = ufl_element.cell().geometric_dimension()
-    tdim = ufl_element.cell().topological_dimension()
+    domain, = ufl_element.domains() # Assuming single domain
+    gdim = domain.geometric_dimension()
+    tdim = domain.topological_dimension()
 
     # Do nothing if we are not in a special case: The special cases
     # occur if we have piola mapped elements (for which value_shape !=
@@ -99,33 +104,57 @@ def needs_oriented_jacobian(form_data):
 def initialize_integral_ir(representation, itg_data, form_data, form_id):
     """Initialize a representation dict with common information that is
     expected independently of which representation is chosen."""
-    entitytype = { "cell": "cell",
+
+    # Mapping from recognized domain types to entity types
+    entity_type = {"cell":           "cell",
                    "exterior_facet": "facet",
                    "interior_facet": "facet",
-                   "point": "vertex",
-                   }[itg_data.domain_type]
-    return { "representation":       representation,
-             "domain_type":          itg_data.domain_type,
-             "domain_id":            itg_data.domain_id,
-             "form_id":              form_id,
-             "rank":                 form_data.rank,
-             "geometric_dimension":  form_data.geometric_dimension,
-             "topological_dimension":form_data.topological_dimension,
-             "entitytype":           entitytype,
-             "num_facets":           cellname_to_num_entities[form_data.cell.cellname()][-2],
-             "num_vertices":         cellname_to_num_entities[form_data.cell.cellname()][0],
-             "needs_oriented":       needs_oriented_jacobian(form_data),
-           }
+                   "point":          "vertex",
+                   "custom":         "cell"}[itg_data.integral_type]
+
+    # Extract data
+    cellname = itg_data.domain.cell().cellname()
+    tdim = itg_data.domain.topological_dimension()
+    assert all(tdim == itg.domain().topological_dimension() for itg in itg_data.integrals)
+
+    # Set number of cells if not set TODO: Get automatically from number of domains
+    num_cells = itg_data.metadata.get("num_cells")
+
+    return {"representation":        representation,
+            "integral_type":         itg_data.integral_type,
+            "subdomain_id":          itg_data.subdomain_id,
+            "form_id":               form_id,
+            "rank":                  form_data.rank,
+            "geometric_dimension":   form_data.geometric_dimension,
+            "topological_dimension": tdim,
+            "entitytype":            entity_type,
+            "num_facets":            cellname_to_num_entities[cellname][-2],
+            "num_vertices":          cellname_to_num_entities[cellname][0],
+            "needs_oriented":        needs_oriented_jacobian(form_data),
+            "num_cells":             num_cells,
+            "enabled_coefficients":  itg_data.enabled_coefficients,
+            }
+
+def generate_enabled_coefficients(enabled_coefficients):
+    # TODO: I don't know how to implement this using the format dict, this will do for now:
+    initializer_list = ", ".join("true" if enabled else "false"
+                                 for enabled in enabled_coefficients)
+    code = '\n'.join([
+        "static const std::vector<bool> enabled({%s});" % initializer_list,
+        "return enabled;",
+        ])
+    return code
 
 def initialize_integral_code(ir, prefix, parameters):
     "Representation independent default initialization of code dict for integral from intermediate representation."
     code = {}
     code["restrict"] = parameters["restrict_keyword"]
-    code["classname"] = format["classname " + ir["domain_type"] + "_integral"](prefix, ir["form_id"], ir["domain_id"])
+    code["classname"] = format["classname " + ir["integral_type"] + "_integral"](prefix, ir["form_id"], ir["subdomain_id"])
     code["members"] = ""
     code["constructor"] = format["do nothing"]
     code["constructor_arguments"] = ""
     code["initializer_list"] = ""
     code["destructor"] = format["do nothing"]
+    code["enabled_coefficients"] = generate_enabled_coefficients(ir["enabled_coefficients"])
     #code["additional_includes_set"] = set() #ir["additional_includes_set"]
     return code
