@@ -16,7 +16,7 @@
 # along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
 # First added:  2010-01-25
-# Last changed: 2010-01-25
+# Last changed: 2014-04-15
 
 # FFC modules
 from ffc.log import debug
@@ -27,29 +27,32 @@ from ffc.tensor.monomialtransformation import transform_monomial_form
 
 def estimate_cost(integral, function_replace_map):
     """
-    Estimate cost of tensor representation for integrand. The cost is
+    Estimate cost of tensor representation for integral. The cost is
     computed as the sum of the number of coefficients and derivatives,
     if the integrand can be represented as a monomial, and -1 if not.
     """
 
-    # TODO: Implement this to take integrand instead of integral? May allow simplification inn caller code.
+    # Check that integral type is supported
+    supported = ["cell", "exterior_facet", "interior_facet"]
+    if not integral.integral_type() in supported:
+        return -1
 
     # Extract monomial integrand
+    integrand = integral.integrand()
     try:
-        monomial_form = extract_monomial_form([integral], function_replace_map)
+        monomial_form = extract_monomial_form([integrand], function_replace_map)
         transform_monomial_form(monomial_form)
     except Exception, exception:
         debug("Monomial extraction failed: " + str(exception))
         return -1
 
-    # Check that we get just one integral
-    if not len(monomial_form.integrals) == 1:
+    # Check that we get just one integrand
+    if not len(monomial_form) == 1:
         error("Expecting just one integrand.")
 
     # Compute cost
-    (integrand, measure) = monomial_form.integrals[0]
     cost = 0
-    for monomial in integrand.monomials:
-        cost = max(cost, len(monomial.coefficients) + len(monomial.transforms))
-
+    for integrand in monomial_form:
+        for monomial in integrand.monomials:
+            cost = max(cost, len(monomial.coefficients) + len(monomial.transforms))
     return cost
