@@ -53,14 +53,14 @@ def _tabulate_tensor(ir, parameters):
 
     # Extract data from intermediate representation
     AK = ir["AK"]
-    domain_type = ir["domain_type"]
+    integral_type = ir["integral_type"]
     tdim = ir["topological_dimension"]
     gdim = ir["geometric_dimension"]
     oriented = ir["needs_oriented"]
     num_facets = ir["num_facets"]
 
     # Check integral type and generate code
-    if domain_type == "cell":
+    if integral_type == "cell":
 
         # Generate code for one single tensor contraction
         t_code = _generate_tensor_contraction(AK, parameters, g_set)
@@ -78,7 +78,7 @@ def _tabulate_tensor(ir, parameters):
         j_code += "\n"
         j_code += format["scale factor snippet"]
 
-    elif domain_type == "exterior_facet":
+    elif integral_type == "exterior_facet":
 
         # Generate code for num_facets tensor contractions
         cases = [None for i in range(num_facets)]
@@ -99,7 +99,7 @@ def _tabulate_tensor(ir, parameters):
         j_code += "\n"
         j_code += format["facet determinant"](tdim, gdim)
 
-    elif domain_type == "interior_facet":
+    elif integral_type == "interior_facet":
 
         # Generate code for num_facets x num_facets tensor contractions
         cases = [[None for j in range(num_facets)] for i in range(num_facets)]
@@ -118,11 +118,13 @@ def _tabulate_tensor(ir, parameters):
             j_code += "\n"
             j_code += format["compute_jacobian_inverse"](tdim, gdim, r=_r)
             j_code += "\n"
+            if oriented:
+                j_code += format["orientation"](tdim, gdim, r=_r)
         j_code += format["facet determinant"](tdim, gdim, r="+")
         j_code += "\n"
 
     else:
-        error("Unhandled integral type: " + str(domain_type))
+        error("Unhandled integral type: " + str(integral_type))
 
     # Remove unused declarations from Jacobian code
     j_code = remove_unused(j_code, j_set)
@@ -346,8 +348,8 @@ def _generate_geometry_tensors(terms, j_set, g_set, tdim, gdim):
             value = format_add(values)
 
             # Multiply with determinant factor
-            det = GK.determinant
-            value = _multiply_value_by_det(value, GK.determinant, len(values) > 1, j_set)
+            dets = GK.determinants
+            value = _multiply_value_by_det(value, dets, len(values) > 1, j_set)
             det_used = True
 
             # Add code
@@ -387,20 +389,34 @@ def _generate_entry(GK, a, i, j_set, tdim, gdim):
 
     return entry
 
-def _multiply_value_by_det(value, det, is_sum, j_set):
-    "Generate code for multiplication of value by determinant."
-    if not det.power == 0:
-        J = format["det(J)"](det.restriction)
-        d = [format["power"](J, det.power)]
-        j_set.add(J)
+def _multiply_value_by_det(value, dets, is_sum, j_set):
+    "Generate code for multiplication of value by determinant(s)."
+
+    # FIXME: MER: This is way complicated than it should be
+
+    # Cell / exterior facets:
+    d = []
+    if all([det.restriction == None for det in dets]):
+        total_power = sum(det.power for det in dets)
+        if not total_power == 0:
+            J = format["det(J)"](None)
+            d += [format["power"](J, total_power)]
+            j_set.add(J)
+    # Interior facets
     else:
-        d = []
+        for det in dets:
+            if not det.power == 0:
+                J = format["det(J)"](det.restriction)
+                d += [format["power"](J, det.power)]
+                j_set.add(J)
+
     if value == "1.0":
         v = []
     elif is_sum:
         v = [format["grouping"](value)]
     else:
         v = [value]
+
     return format["multiply"](d + [format["scale factor"]] + v)
 
 def _extract_factors(GK, a, b, j_set, tdim, gdim, index_type):
