@@ -9,23 +9,22 @@
 #
 # FFC is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with FFC.  If not, see <http://www.gnu.org/licenses/>.
+# along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
 # First added:  2010-01-18
 # Last changed: 2010-01-18
 
 from cppcode import evaluate_basis_code
 from ufl import FiniteElement, MixedElement
+from instant.output import get_status_output
 
-import sys, os, commands, pickle, numpy, shutil
+import sys, os, pickle, numpy, shutil
 
 # Elements, supported by FFC and FIAT, and their supported shape and orders
-# TODO: RT order 0 gives error from FIAT, but is allowed by UFL
-# TODO: Nedelec order 0 gives error from FIAT, but is allowed by UFL
 single_elements = [ {"family": "Lagrange",\
                       "shapes": ["interval", "triangle", "tetrahedron"],\
                       "orders": [1, 2, 3, 4]},\
@@ -37,16 +36,22 @@ single_elements = [ {"family": "Lagrange",\
                       "orders": [1]},\
                     {"family": "Raviart-Thomas",\
                       "shapes": ["triangle", "tetrahedron"],\
-                      "orders": [1, 2]},\
+                      "orders": [1, 2, 3]},\
+                    {"family": "Discontinuous Raviart-Thomas",\
+                      "shapes": ["triangle", "tetrahedron"],\
+                      "orders": [1, 2, 3]},\
                     {"family": "Brezzi-Douglas-Marini",\
                       "shapes": ["triangle", "tetrahedron"],\
                       "orders": [1, 2, 3]},\
                     {"family": "Brezzi-Douglas-Fortin-Marini",\
-                      "shapes": ["triangle", "tetrahedron"],\
-                      "orders": [1, 2, 3]},\
+                      "shapes": ["triangle"],\
+                      "orders": [2]},\
                     {"family": "Nedelec 1st kind H(curl)",\
                       "shapes": ["triangle", "tetrahedron"],\
-                      "orders": [1, 2]}]
+                      "orders": [1, 2, 3]},\
+                    {"family": "Nedelec 2nd kind H(curl)",\
+                      "shapes": ["triangle", "tetrahedron"],\
+                      "orders": [1, 2, 3]}]
 
 # Create some mixed elements
 dg0_tri = FiniteElement("DG", "triangle", 0)
@@ -54,6 +59,7 @@ dg1_tri = FiniteElement("DG", "triangle", 1)
 cg1_tri = FiniteElement("CG", "triangle", 1)
 cr1_tri = FiniteElement("CR", "triangle", 1)
 rt1_tri = FiniteElement("RT", "triangle", 1)
+drt2_tri = FiniteElement("DRT", "triangle", 2)
 bdm1_tri = FiniteElement("BDM", "triangle", 1)
 ned1_tri = FiniteElement("N1curl", "triangle", 1)
 
@@ -62,6 +68,7 @@ dg1_tet = FiniteElement("DG", "tetrahedron", 1)
 cg1_tet = FiniteElement("CG", "tetrahedron", 1)
 cr1_tet = FiniteElement("CR", "tetrahedron", 1)
 rt1_tet = FiniteElement("RT", "tetrahedron", 1)
+drt2_tet = FiniteElement("DRT", "tetrahedron", 2)
 bdm1_tet = FiniteElement("BDM", "tetrahedron", 1)
 ned1_tet = FiniteElement("N1curl", "tetrahedron", 1)
 
@@ -69,10 +76,12 @@ mixed_elements = [MixedElement([dg0_tri]*4), MixedElement([cg1_tri]*3), MixedEle
                   MixedElement([dg1_tri, cg1_tri, cr1_tri, rt1_tri, bdm1_tri, ned1_tri]),\
                   MixedElement([MixedElement([rt1_tri, cr1_tri]), cg1_tri, ned1_tri]),\
                   MixedElement([ned1_tri, dg1_tri, MixedElement([rt1_tri, cr1_tri])]),\
+                  MixedElement([drt2_tri, cg1_tri]),\
                   MixedElement([dg0_tet]*4), MixedElement([cg1_tet]*3), MixedElement([bdm1_tet]*2),\
                   MixedElement([dg1_tet, cg1_tet, cr1_tet, rt1_tet, bdm1_tet, ned1_tet]),\
                   MixedElement([MixedElement([rt1_tet, cr1_tet]), cg1_tet, ned1_tet]),\
-                  MixedElement([ned1_tet, dg1_tet, MixedElement([rt1_tet, cr1_tet])])]
+                  MixedElement([ned1_tet, dg1_tet, MixedElement([rt1_tet, cr1_tet])]),\
+                  MixedElement([drt2_tet, cg1_tet])]
 
 ffc_failed = []
 gcc_failed = []
@@ -129,7 +138,7 @@ def compile_element(ufl_element):
     if isinstance(ufl_element, (FiniteElement, MixedElement)):
         f.write("element = " + repr(ufl_element))
     f.close()
-    error, out = commands.getstatusoutput("ffc test.ufl")
+    error, out = get_status_output("ffc test.ufl")
     if error:
         ffc_failed.append(repr(ufl_element))
     return error
@@ -164,15 +173,18 @@ def compute_values(ufl_element):
     f.write(code)
     f.close()
 
+    # Get UFC flags
+    ufc_cflags = get_status_output("pkg-config --cflags ufc-1")[1].strip()
+
     # Compile g++ code
-    c = "g++ `pkg-config --cflags ufc-1` -Wall -Werror -o evaluate_basis evaluate_basis.cpp"
-    error, output = commands.getstatusoutput(c)
+    c = "g++ %s -Wall -Werror -o evaluate_basis evaluate_basis.cpp" % ufc_cflags
+    error, output = get_status_output(c)
     if error:
         gcc_failed.append(repr(ufl_element))
         return None
 
     # Run compiled code and get values
-    error, output = commands.getstatusoutput("./evaluate_basis")
+    error, output = get_status_output(".%sevaluate_basis" % os.path.sep)
     if error:
         run_failed.append(repr(ufl_element))
         return None
