@@ -19,74 +19,17 @@
 # Last changed: 2013-01-31
 
 from cppcode import evaluate_basis_code_fiat
-from ufl import FiniteElement, MixedElement
+from ufl import FiniteElement
 from ffc.fiatinterface import create_element, reference_cell
 from ffc.mixedelement import MixedElement as FFCMixedElement
 from ffc.log import info, error, debug
 import numpy
 import sys, os, numpy, shutil
-from test_common import xcomb, compile_element, print_results, compile_gcc_code,\
+from test_common import compile_element, print_results, compile_gcc_code,\
                         run_code, get_element_name, verify_values
 import time
 from ffc.log import push_level, pop_level, CRITICAL, INFO
-
-# Elements, supported by FFC and FIAT, and their supported shape and degrees
-# TODO: RT degree 0 gives error from FIAT, but is allowed by UFL
-# TODO: Nedelec degree 0 gives error from FIAT, but is allowed by UFL
-single_elements = [
-                    {"family": "Lagrange",\
-                      "shapes": ["interval", "triangle", "tetrahedron"],\
-                      "degrees": [1, 2, 3, 4]},\
-                    {"family": "Discontinuous Lagrange",\
-                      "shapes": ["interval", "triangle", "tetrahedron"],\
-                      "degrees": [0, 1, 2, 3, 4]},\
-                    {"family": "Crouzeix-Raviart",\
-                      "shapes": ["triangle", "tetrahedron"],\
-                      "degrees": [1]},\
-                    {"family": "Raviart-Thomas",\
-                      "shapes": ["triangle", "tetrahedron"],\
-                      "degrees": [1, 2, 3]},\
-                    {"family": "Brezzi-Douglas-Marini",\
-                      "shapes": ["triangle", "tetrahedron"],\
-                      "degrees": [1, 2, 3]},\
-                    {"family": "Nedelec 1st kind H(curl)",\
-                      "shapes": ["triangle", "tetrahedron"],\
-                      "degrees": [1, 2, 3]}
-# Not supported in new FIAT
-#                    {"family": "Brezzi-Douglas-Fortin-Marini",\
-#                      "shapes": ["triangle", "tetrahedron"],\
-#                      "degrees": [1, 2, 3]},\
-]
-
-# Create some mixed elements
-dg0_tri = FiniteElement("DG", "triangle", 0)
-dg1_tri = FiniteElement("DG", "triangle", 1)
-cg1_tri = FiniteElement("CG", "triangle", 1)
-cr1_tri = FiniteElement("CR", "triangle", 1)
-rt1_tri = FiniteElement("RT", "triangle", 1)
-bdm1_tri = FiniteElement("BDM", "triangle", 1)
-ned1_tri = FiniteElement("N1curl", "triangle", 1)
-
-dg0_tet = FiniteElement("DG", "tetrahedron", 0)
-dg1_tet = FiniteElement("DG", "tetrahedron", 1)
-cg1_tet = FiniteElement("CG", "tetrahedron", 1)
-cr1_tet = FiniteElement("CR", "tetrahedron", 1)
-rt1_tet = FiniteElement("RT", "tetrahedron", 1)
-bdm1_tet = FiniteElement("BDM", "tetrahedron", 1)
-ned1_tet = FiniteElement("N1curl", "tetrahedron", 1)
-
-# Create combinations in pairs.
-mix_tri = [MixedElement(e) for e in xcomb([dg0_tri, dg1_tri, cg1_tri, cr1_tri, rt1_tri, bdm1_tri, ned1_tri], 2)]
-mix_tet = [MixedElement(e) for e in xcomb([dg0_tet, dg1_tet, cg1_tet, cr1_tet, rt1_tet, bdm1_tet, ned1_tet], 2)]
-
-mixed_elements = [MixedElement([dg0_tri]*4), MixedElement([cg1_tri]*3), MixedElement([bdm1_tri]*2),\
-                  MixedElement([dg1_tri, cg1_tri, cr1_tri, rt1_tri, bdm1_tri, ned1_tri]),\
-                  MixedElement([MixedElement([rt1_tri, cr1_tri]), cg1_tri, ned1_tri]),\
-                  MixedElement([ned1_tri, dg1_tri, MixedElement([rt1_tri, cr1_tri])]),\
-                  MixedElement([dg0_tet]*4), MixedElement([cg1_tet]*3), MixedElement([bdm1_tet]*2),\
-                  MixedElement([dg1_tet, cg1_tet, cr1_tet, rt1_tet, bdm1_tet, ned1_tet]),\
-                  MixedElement([MixedElement([rt1_tet, cr1_tet]), cg1_tet, ned1_tet]),\
-                  MixedElement([ned1_tet, dg1_tet, MixedElement([rt1_tet, cr1_tet])])] + mix_tri + mix_tet
+from elements import single_elements, mixed_elements
 
 # Some random points
 random_points = {1: [(0.114,), (0.349,), (0.986,)],
@@ -116,13 +59,14 @@ def get_data(ufl_element):
         deriv_order = element.degree()
 
     # Get coordinates of the reference cell.
-    ref_coords = reference_cell(ufl_element.cell().cellname()).get_vertices()
+    domain, = ufl_element.domains()
+    ref_coords = reference_cell(domain.cell().cellname()).get_vertices()
 
     # Get the locations of the fiat element dofs.
     elem_points =  [L.pt_dict.keys()[0] for L in element.dual_basis()]
 
     # Add some random points.
-    geo_dim = ufl_element.cell().geometric_dimension()
+    geo_dim = domain.geometric_dimension()
     points = elem_points + random_points[geo_dim]
 
     return (element, points, geo_dim, ref_coords, deriv_order)
@@ -282,8 +226,8 @@ def main(debug_level):
     elements = []
     for element in single_elements:
         for shape in element["shapes"]:
-            for degree in element["degrees"]:
-                elements.append(FiniteElement(element["family"], shape, degree))
+            for order in element["orders"]:
+                elements.append(FiniteElement(element["family"], shape, order))
 
     # Add the mixed elements
     elements += mixed_elements
